{% set button_primary_color=button_primary_color|default("#001C3C") %}
{% set button_primary_text_color=button_primary_text_color|default("#ff6201") %}
<div class="sm_90_percent_width"
	style="padding-right: 30px; padding-left: 30px; margin-bottom: 30px;"
>
	<table
		style="width: 100%; text-align: center;"
		class="sm_table"
	>
		<tr style="width: 100%;">
			<td style="width: 100%;">
				<span
					style="display: inline-block; position:relative; border-radius:4px; background-color:{{ button_primary_color }}; "
					class="button_link_wrapper plum sm_full_width"
				>
					<a
						class="button_link sm_full_width sm_border_box plum"
						href="{{ url }}"
						style="border-top:13px solid; border-bottom:13px solid; border-right:24px solid; border-left:24px solid; border-color:{{ button_primary_color }}; border-radius:4px; background-color:{{ button_primary_color }}; color:{{ button_primary_text_color }}; font-size:16px; font-weight:600; line-height:18px; word-break:break-word;  display: inline-block; text-align: center; text-decoration: none !important;"
					>
						{{ content|safe }}
					</a>
				</span>
			</td>
		</tr>
	</table>
</div>
